<?php


namespace Drupal\legalweb_cloud;


use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Http\ClientFactory;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Url;
use GuzzleHttp\Exception\GuzzleException;

class LWCManager {

  const SETTINGS = 'legalweb_cloud.settings';
  const API_URL = 'https://legalweb.io/api';
  const RESPONSE_PATH = 'private://legalweb_cloud/';
  const RESPONSE_FILENAME = 'response.json';
  const ASSET_PATH = 'public://legalweb_cloud/';
  const ASSET_FILENAME = 'legalweb_cloud';

  protected $callback_url = '';
  protected $legalweb_cloud_settings = [];

  protected $settings;
  protected $http_factory;
  protected $state;
  protected $file_system;


  public function __construct(ConfigFactoryInterface $config_factory,
                              ClientFactory $http_client_factory,
                              StateInterface $state,
                              FileSystemInterface $file_system) {
    $this->settings = $config_factory->get(LWCManager::SETTINGS);
    $this->http_factory = $http_client_factory;
    $this->state = $state;
    $this->file_system = $file_system;
    $this->getRemoteSettings();
  }

  /**
   * Get the Legalweb.io Configuration and save it in the private folder in
   * the local filesystem
   *
   * @param false $force
   *
   * @return bool
   */
  public function requestRemoteSettings($force = FALSE) {
    $now = time();
    if (!$force) {
      // check State API when the last update request was less than 24h ago,
      // skip if $force = TRUE
      $interval = $this->settings->get('interval') ?? 86400;
      $last_update = $this->state->get('legalweb_cloud.last_update') ?? 0;

      if (($now - $last_update) < $interval) {
        return FALSE;
      }
    }

    $http_client = $this->http_factory->fromOptions([
      'base_uri' => self::API_URL,
      'verify' => TRUE,
      'timeout' => 30,
      'headers' => [
        'content-type' => 'application/x-www-form-urlencoded',
        'Guid' => $this->settings->get('guid'),
        'Callback' => $this->getCallbackUrl(),
      ]
    ]);

    try {
      $response = $http_client->request('GET');
    } catch (GuzzleException $e) {
      // TODO: log request error
      return FALSE;
    }

    $response_json = $response->getBody();

    // Request is invalid if a code is returned
    if (!empty($response_json->code)) {
      // TODO: log ->code and ->message
      return FALSE;
    }

    $response_abs_path = self::RESPONSE_PATH . self::RESPONSE_FILENAME;
    if (file_exists($response_abs_path)) {
      $this->file_system->copy($response_abs_path,
        $response_abs_path . '.old',
        FileSystemInterface::EXISTS_REPLACE );
    }

    $directory = self::RESPONSE_PATH;
    $this->file_system->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY);
    $this->file_system->saveData(
      $response_json,
      $response_abs_path,
      FileSystemInterface::EXISTS_REPLACE
    );

    $this->state->set('legalweb_cloud.last_update', $now);

    $this->generateAssets(
      json_decode($response_json)
    );

    return TRUE;
  }

  /**
   * Callback URL for legalweb.io to force an API Call from their side.
   * Will be forced always.
   *
   * @return array
   */
  public function callbackConfiguration() {
    $this->requestRemoteSettings(TRUE);
    return [];
  }

  /**
   * Seperates the response.json into different files which will
   * be used by the library 'generated' defined in the libraries.yml
   *
   * @param null $response
   *
   * @return bool
   */
  public function generateAssets($response = NULL) {
    // If no response data is passed, read from current reponse.json
    // in the local file system.
    if (empty($response)) {
      $response_abs_path = self::RESPONSE_PATH . self::RESPONSE_FILENAME;
      if (file_exists($response_abs_path)) {
        $response = json_decode(file_get_contents($response_abs_path));
      }
      else {
        // TODO log error
        return FALSE;
      }
    }

    /**
     * Generate CSS file in the public files folder
     * TODO: check if anything has changed -> trigger cache something
     */
    $asset_directory  = self::ASSET_PATH;
    $this->file_system->prepareDirectory($asset_directory, FileSystemInterface::CREATE_DIRECTORY);
    $this->file_system->saveData(
      $response->services->dppopupcss,
      $asset_directory . self::ASSET_FILENAME . '.css',
      FileSystemInterface::EXISTS_REPLACE
    );

    /**
     * Generate JS file in the public files folder
     * Encapsulate in Drupal Behaviour
     * TODO: check if anything has changed -> trigger cache something
     */
    $legalweb_cloud_js = $response->services->dppopupjs;
    $general_config = json_encode($response->services->dppopupconfig->spDsgvoGeneralConfig);
    $integration_config = json_encode($response->services->dppopupconfig->spDsgvoIntegrationConfig);
    $js  = <<< JS
      var spDsgvoGeneralConfig = JSON.parse('$general_config');
      var spDsgvoIntegrationConfig = JSON.parse('$integration_config');

      $legalweb_cloud_js
    JS;
    $this->file_system->saveData(
      $js,
      $asset_directory . self::ASSET_FILENAME . '.js',
      FileSystemInterface::EXISTS_REPLACE
    );

    /**
     * Generate the $this->remote_config as a file.
     * It is a slimmed down version of response.json, without the CSS & JS
     * from above.
     * TODO: check if anything has changed -> trigger cache something
     */
    unset($response->services->dppopupjs,
      $response->services->dppopupcss,
      $response->services->dppopupconfig->spDsgvoGeneralConfig,
      $response->services->dppopupconfig->spDsgvoIntegrationConfig);
    $legalweb_cloud_settings = json_encode($response);
    $this->file_system->saveData(
      $legalweb_cloud_settings,
      $asset_directory . self::ASSET_FILENAME . '.json',
      FileSystemInterface::EXISTS_REPLACE
    );

    // TODO: possible to delete only those for the module?
    _drupal_flush_css_js();
    \Drupal::service('asset.css.collection_optimizer')->deleteAll();
    \Drupal::service('asset.js.collection_optimizer')->deleteAll();

    return TRUE;
  }

  /**
   * Returns all or the value of given key from Legalweb s JSON
   *
   * @param null $key
   *
   * @return array|null
   */
  public function getRemoteSettings() {
    if (!empty($this->legalweb_cloud_settings)) {
      return $this->legalweb_cloud_settings;
    }

    $legalweb_cloud_settings_path = self::ASSET_PATH . self::ASSET_FILENAME . '.json';
    if (file_exists($legalweb_cloud_settings_path)) {
      $this->legalweb_cloud_settings = json_decode(
        file_get_contents($legalweb_cloud_settings_path)
      );
    }

    return $this->legalweb_cloud_settings;
  }

  /**
   * Generate Callback URL
   *
   * @return \Drupal\Core\GeneratedUrl|string
   */
  public function getCallbackUrl() {
    if (empty($this->callback_url)) {
      try {
        $url = Url::fromRoute('legalweb_cloud.callback_page', [], ['absolute' => TRUE]);
        $this->callback_url = $url->toString();
      }
      catch (\Exception $exception) {
        // Happens during install once
      }
    }
    return $this->callback_url;
  }

  /**
   * Current mappings for the remote configuration
   */
  public function getDomainId() {
    return $this->legalweb_cloud_settings->domain->domain_id ?? NULL;
  }

  public function getDomain() {
    return $this->legalweb_cloud_settings->domain->domain_url ?? NULL;
  }

  public function getImprintHTML($langcode = NULL) {
    return $this->getServiceHTML('imprint', $langcode);
  }

  public function getDataPrivacyHTML($langcode = NULL) {
    return $this->getServiceHTML('dpstatement', $langcode);
  }

  public function getPopupHTML($langcode = NULL) {
    return $this->getServiceHTML('dppopup', $langcode);
  }

  protected function getServiceHTML($service, $langcode = NULL) {
    $html = NULL;
    if ($langcode &&
      !empty($this->legalweb_cloud_settings->services->$service)) {
      $html = $this->legalweb_cloud_settings->services->$service->$langcode ?? NULL;
    }

    if (!$html && !empty($this->legalweb_cloud_settings->services->$service)) {
      foreach ($this->legalweb_cloud_settings->services->$service as $key => $value) {
        $html = $value;
        break;
      }
    }

    return $html;
  }

}